import { error } from '@sveltejs/kit';
import { Users, type UserSession } from './users';
import { hasPrivileges, type RequiredPrivileges } from '$lib/utils';

export class AdminUtils {
	/**
	 * Check a session privileges against a list of required privileges.
	 * @param session User session with privileges loaded
	 * @param privileges OR logic, AND's arrays
	 * @throws 403 error if privileges are not met
	 */
	static checkPrivileges(session: UserSession, privileges: RequiredPrivileges): void {
		if (!session.privileges?.length || !hasPrivileges(session.privileges, privileges)) {
			error(403, 'Forbidden resource');
		}
	}

	/**
	 * Get the user who is committing an action.
	 * @param locals App locals
	 * @param privileges Required privileges for action
	 * @returns Current user and their session cookie
	 */
	static async getActionUser(locals: App.Locals, privileges: RequiredPrivileges) {
		const userSession = locals.session.data?.user;
		const currentUser = await Users.getBySession(userSession);
		if (!userSession || !currentUser) {
			return error(403);
		}

		userSession.privileges = await Users.getUserPrivileges(currentUser);
		AdminUtils.checkPrivileges(userSession, privileges);

		return { currentUser, userSession };
	}
}
