import { env } from '$env/dynamic/private';
import { drizzle } from 'drizzle-orm/mysql2';
import mysql from 'mysql2';
import * as schema from './schema';
import { Logger } from '../logger';

const { DATABASE_DB, DATABASE_HOST, DATABASE_PASS, DATABASE_USER } = env;

export class DB {
	static mysqlConnection: mysql.Pool;
	static drizzle: ReturnType<typeof drizzle<typeof schema>>;

	static async init() {
		DB.mysqlConnection = mysql.createPool({
			host: DATABASE_HOST,
			user: DATABASE_USER,
			password: DATABASE_PASS,
			database: DATABASE_DB,
			keepAliveInitialDelay: 10 * 1000, // 10 seconds
			enableKeepAlive: true,
			maxIdle: 0,
			idleTimeout: 5 * 60 * 1000 // 5 minutes
		});

		DB.mysqlConnection.on('connection', (connection) => {
			connection.on('error', (error) => {
				// We log warning on connection error, this is not a critical error
				Logger.warn('Error received on connection. Will destroy.', { error });
				connection.destroy();
			});
		});

		DB.drizzle = drizzle({ client: DB.mysqlConnection, schema, mode: 'default' });

		Logger.debug('Database initialization complete');
	}
}

export * from './schema';
