import { mkdir, readFile, rm, stat, unlink, writeFile } from 'fs/promises';
import { dirname, join, resolve } from 'path';

// TODO: implement S3

export class FileBackend {
	static async fileExists(path: string | string[]) {
		try {
			await stat(FileBackend.filePath(path));
			return true;
		} catch {
			return false;
		}
	}

	static async saveFile(path: string | string[], contents: Buffer) {
		const fullPath = FileBackend.filePath(path);
		try {
			await mkdir(dirname(fullPath), { recursive: true });
		} catch {}
		return writeFile(fullPath, contents);
	}

	static async readFile<T = Buffer>(path: string | string[], encoding?: BufferEncoding) {
		return readFile(FileBackend.filePath(path), { encoding }) as T;
	}

	static async deleteFile(path: string | string[]) {
		try {
			await unlink(FileBackend.filePath(path));
			return true;
		} catch {
			return false;
		}
	}

	static async deleteDirectory(path: string | string[]) {
		try {
			await rm(FileBackend.filePath(path), { recursive: true });
			return true;
		} catch {
			return false;
		}
	}

	private static filePath(path: string | string[]) {
		return resolve(Array.isArray(path) ? join(...path) : path);
	}

	public static async init() {
		// stub
	}
}
