import { browser } from '$app/environment';
import { locale, t } from '$lib/i18n';
import { derived, writable } from 'svelte/store';

export interface Message {
	text: string;
	type: 'success' | 'error';
	actions?: {
		label: string;
		action: () => void;
	}[];
}

export const messages = writable<Message[]>([]);

export const dismissMessage = (msg: Message) =>
	messages.update((current) => current.filter((entry) => entry !== msg));

export const displayMessage = (msg: Message) =>
	browser && messages.update((current) => [...current, msg]);

export const clearMessages = () => messages.set([]);

export const translateError = {
	...derived([t, locale], ([tFn]) => (error: string, prefix = 'common.errors') => {
		const [keyword, args] = error.split('(');
		const argList = args
			?.substring(0, args.length - 1)
			.split(',')
			.reduce<Record<string, string>>(
				(data, value, i) => ({ ...data, [`arg${i}`]: value.trim() }),
				{}
			);
		return tFn(`${prefix}.${keyword}`, argList);
	})
};
