import { OAuth2Error, SlowDown } from '$lib/server/oauth2/error.js';
import { OAuth2Response } from '$lib/server/oauth2/response.js';
import { OAuth2PushedAuthorizationController } from '$lib/server/oauth2/controller/pushed-authorization.js';
import { RateLimiter } from 'sveltekit-rate-limiter/server';
import { Audit, AuditAction } from '$lib/server/audit';

const limiter = new RateLimiter({
	IP: [15, 'm']
});

export const POST = async (event) => {
	const { request, url } = event;

	try {
		if (await limiter.isLimited(event)) {
			await Audit.insertRequest(AuditAction.THROTTLE, event, undefined, `oauth2 par attempt`);
			throw new SlowDown('Please, slow down!');
		}

		return await OAuth2PushedAuthorizationController.postRequest({ request });
	} catch (error) {
		if (error instanceof OAuth2Error) {
			return OAuth2Response.error(url, error);
		}
		throw error;
	}
};
