import { AdminUtils } from '$lib/server/admin-utils';
import { Audit } from '$lib/server/audit/audit.js';
import { AuditAction } from '$lib/server/audit/types.js';
import { Changesets } from '$lib/server/changesets.js';

const PAGE_SIZE = 50;

interface AuditSearchParams {
	page: string;
	pageSize: string;
	actions: string;
	user: string;
	content: string;
	ip: string;
	flagged: string;
}

export const load = async ({ url, parent }) => {
	const { user: userInfo } = await parent();
	AdminUtils.checkPrivileges(userInfo, ['admin:audit']);

	const actions = url.searchParams.getAll('actions') as AuditAction[];
	const { page, pageSize, user, content, ip, flagged } = Changesets.only(
		['page', 'pageSize', 'user', 'content', 'ip', 'flagged'],
		url.searchParams
	);

	const limit = Number(pageSize) || PAGE_SIZE;
	const offset = ((Number(page) || 1) - 1) * limit;

	const data = await Audit.searchAudit({
		limit,
		offset,
		actions,
		user,
		content,
		ip,
		flagged: !!flagged
	});

	return {
		...data,
		actions: Object.values(AuditAction)
	};
};
